@@ -45,7 +45,7 @@
 #'
 #' @name mlg
 #'
-#' @param pop a \code{\link{genind}} object.
+#' @param pop a \code{\linkS4class{genind}} or \code{\linkS4class{genclone}} object.
 #'
 #' @param sublist a \code{vector} of population names or indices that the user
 #' wishes to keep. Default to "ALL".
@@ -73,11 +73,69 @@
 #'
 #' @param total \code{logical} If \code{TRUE}, a row containing the sum of all
 #' represented MLGs is appended to the matrix produced by mlg.table.
+#' 
+#' @return 
+#' \subsection{mlg}{
+#' an integer describing the number of multilocus genotypes observed.
+#' }
+#' \subsection{mlg.table}{
+#' a matrix with columns indicating unique multilocus genotypes and rows
+#' indicating populations.
+#' }
+#' \subsection{mlg.vector}{
+#' a numeric vector naming the multilocus genotype of each individual in
+#'   the dataset.
+#' }
+#' \subsection{mlg.crosspop}{
+#' \itemize{
+#' \item{default}{ a \code{list} where each element contains a named integer vector representing the number of individuals represented from each population in that MLG}
+#' \item{\code{indexreturn = TRUE}}{ a \code{vector} of integers defining the multilocus genotypes that have individuals crossing populations}
+#' \item{\code{df = TRUE}}{ A long form data frame with the columns: MLG, Population, Count. Useful for graphing with ggplot2}
+#' }
+#' }
+#' \subsection{mlg.id}{
+#' a list of multilocus genotypes with the associated individual names per MLG.
+#' }
 #'
-#' @seealso \code{\link{diversity}} \code{\link{popsub}}
+#' @seealso \code{\link[vegan]{diversity}} \code{\link{popsub}}
 #' @author Zhian N. Kamvar
 #' @examples
 #'
+#' # Load the data set
+#' data(Aeut)
+#' 
+#' # Investigate the number of multilocus genotypes.
+#' amlg <- mlg(Aeut)
+#' amlg # 119
+#' 
+#' # show the multilocus genotype vector 
+#' avec <- mlg.vector(Aeut)
+#' avec 
+#' 
+#' # Get a table
+#' atab <- mlg.table(Aeut, bar = FALSE)
+#' atab
+#' 
+#' # See where multilocus genotypes cross populations
+#' acrs <- mlg.crosspop(Aeut) # MLG.59: (2 inds) Athena Mt. Vernon
+#' 
+#' # See which individuals belong to each MLG
+#' aid <- mlg.id(Aeut)
+#' aid["59"] # individuals 159 and 57
+#' 
+#' \dontrun{
+#' 
+#' # A simple example. 10 individuals, 5 genotypes.
+#' mat1 <- matrix(ncol=5, 25:1)
+#' mat1 <- rbind(mat1, mat1)
+#' mat <- matrix(nrow=10, ncol=5, paste(mat1,mat1,sep="/"))
+#' mat.gid <- df2genind(mat, sep="/")
+#' mlg(mat.gid)
+#' mlg.vector(mat.gid)
+#' mlg.table(mat.gid)
+#' 
+#' # Now for a more complicated example.
+#' # Data set of 1903 samples of the H3N2 flu virus genotyped at 125 SNP loci.
 #' data(H3N2)
 #' mlg(H3N2, quiet=FALSE)
 #' 
@@ -90,7 +148,6 @@
 #' # Show which genotypes exist accross populations in the entire dataset.
 #' res <- mlg.crosspop(H3N2, quiet=FALSE)
 #'
-#' \dontrun{
 #' # Let's say we want to visualize the multilocus genotype distribution for the
 #' # USA and Russia
 #' mlg.table(H3N2, sublist=c("USA", "Russia"), bar=TRUE)
@@ -110,178 +167,98 @@
 #' # analyze only the MLGs that are duplicated across populations. 
 #' new.H <- H3N2[H.vec %in% inds, ]
 #' 
-#' # A simple example. 10 individuals, 5 genotypes.
-#' mat1 <- matrix(ncol=5, 25:1)
-#' mat1 <- rbind(mat1, mat1)
-#' mat <- matrix(nrow=10, ncol=5, paste(mat1,mat1,sep="/"))
-#' mat.gid <- df2genind(mat, sep="/")
-#' mlg(mat.gid)
-#' mlg.vector(mat.gid)
-#' mlg.table(mat.gid)
 #' }
 NULL
 #==============================================================================#
 #' @rdname mlg
-# Multi Locus Genotype
-#
-# Count the number of unique multilocus genotypes found within a genind object.
-#
-# @param x a \code{\link{genind}} object.
-#
-# @param quiet default \code{TRUE}. If set to \code{FALSE}, it will display the
-# number of individuals and MLG on the output device.
 #'
-#' @return an integer of the number of multilocus genotypes within the sample.
 #'
 #' @export
 #==============================================================================#
 
 mlg <- function(pop, quiet=FALSE){
-  if(!is.genind(pop)){
-    stop("x is not a genind object")
+  if (!is.genind(pop)){
+    stop(paste(substitute(pop), "is not a genind object"))
   }
-  if(nrow(pop@tab)==1){
-    derp <- 1
-  }
-  else {
-    derp <- nrow(unique(pop@tab[,1:ncol(pop@tab)]))
+  if (is.genclone(pop)){
+    out <- length(unique(pop@mlg))
+  } else {
+    if(nrow(pop@tab) == 1){
+      out <- 1
+    }
+    else {
+      out <- nrow(unique(pop@tab[, 1:ncol(pop@tab)]))
+    } 
   } 
   if(quiet!=TRUE){
     cat("#############################\n")
-    cat("# Number of Individuals: "); cat(length(pop@ind.names),"\n")
-    cat("# Number of MLG: "); cat(derp,"\n")
+    cat("# Number of Individuals: ", nInd(pop), "\n")
+    cat("# Number of MLG: ", out, "\n")
     cat("#############################\n")
   }
-  return(derp)
+  return(out)
 }
 #==============================================================================#
-#' @rdname mlg
-# 
-#' @return a matrix with columns indicating unique multilocus genotypes and rows
-#' indicating populations. 
+#' @rdname mlg 
 #'
 #' @note The resulting matrix of \code{mlg.table} can be used for analysis with 
 #' the \code{\link{vegan}} package.
-#' The names of the multilocus genotypes represented will be those from
-#' the entire dataset. If you wish to view those relative to a subsetted
-#' dataset, you can use \code{mlg.bar(popsub(pop, ...))}.
 #' 
 #' @export
-#
-#
 #==============================================================================#
 mlg.table <- function(pop, sublist="ALL", blacklist=NULL, mlgsub=NULL, bar=TRUE, 
                       total=FALSE, quiet=FALSE){  
-  if(!is.genind(pop)){
+  if (!is.genind(pop)){
     stop("This function requires a genind object.")
   }
   mlgtab <- mlg.matrix(pop)
-  if(!is.null(mlgsub)){
+  if (!is.null(mlgsub)){
     mlgtab <- mlgtab[, mlgsub]
-    mlgtab <- mlgtab[which(rowSums(mlgtab) > 0), ]
+    mlgtab <- mlgtab[which(rowSums(mlgtab) > 0L), ]
     pop <- popsub(pop, sublist=rownames(mlgtab))
   }
-  if(sublist[1] != "ALL" | !is.null(blacklist)){
+  if (sublist[1] != "ALL" | !is.null(blacklist)){
     pop <- popsub(pop, sublist, blacklist)
     mlgtab <- mlgtab[unlist(vapply(pop@pop.names, 
-                function(x) which(rownames(mlgtab)==x), 1)), , drop=FALSE]
+                function(x) which(rownames(mlgtab) == x), 1)), , drop=FALSE]
+    rows <- rownames(mlgtab)
   }
-  if(total==TRUE & (nrow(mlgtab) > 1 | !is.null(nrow(mlgtab)) )){
+  if (total==TRUE & (nrow(mlgtab) > 1 | !is.null(nrow(mlgtab)) )){
     mlgtab <- rbind(mlgtab, colSums(mlgtab))
     rownames(mlgtab)[nrow(mlgtab)] <- "Total"
   }
-  #````````````````````````````````````````````````````````````````````````````#
+
   # Dealing with the visualizations.
-  if(bar){
-    # if(!require(ggplot2)){
-    #   warning("ggplot2 must be installed to visualize the MLG distributions.")
-    #   mlgtab <- mlgtab[, which(colSums(mlgtab) > 0)]
-    #   return(mlgtab)
-    # }
-    
-    # Function for setting up and organizing data frame to produce ggplot2 graphs
-    plot1 <- function(mlgt){
-
-      # create a data frame that ggplot2 can read.
-      mlgt.df <- as.data.frame(list(MLG = rep(colnames(mlgt), mlgt), 
-            count = rep(mlgt, mlgt)))
-
-      # Organize the data frame by count in descending order.
-      mlgt.df[["MLG"]] <- reorder(mlgt.df[["MLG"]], -mlgt.df[["count"]])
-
-      # plot it
-      return(ggplot(mlgt.df, aes_string(x = "MLG")) + geom_bar(aes_string(fill = "count"), position="identity"))
-      #theme(axis.text.x=element_text(size = 10, angle=-45, hjust=0)))
-    }
-
+  if (bar){
     # If there is a population structure
     if(!is.null(pop@pop.names)){
       popnames <- pop@pop.names
-      if(total & nrow(mlgtab) > 1)
-        popnames[length(popnames)+1] <- "Total"
-      
-      # Function for printing plots with population structures one by one.  
-      printplot <- function(n, quiet=quiet) {
-        if(!quiet) cat("|",n,"\n")
-
-        # Gather all nonzero values
-        mlgt <- mlgtab[n, mlgtab[n, ] > 0, drop=FALSE]
-
-        # controlling for the situation where the population size is 1.
-        if (sum(mlgtab[n, ]) > 1){ 
-          print(plot1(mlgt) +
-                  theme_classic() %+replace%
-                  theme(axis.text.x=element_text(size = 10, angle=-45, hjust=0, vjust=1)) + 
-                  labs(title=paste("Population:",n,"\nN =",sum(mlgtab[n, ]),
-                                   "MLG =",length(mlgt))))
-        }
+      if(total & nrow(mlgtab) > 1){
+        popnames[length(popnames) + 1] <- "Total"
       }
-      
       # Apply this over all populations. 
-      invisible(lapply(popnames, printplot, quiet=quiet))
-    }
-    
-    # If there is no population structure detected.
-    else {
-      print(plot1(mlgtab) + 
-              theme_classic() %+replace%
-              theme(axis.text.x=element_text(size = 10, angle=-45, hjust=0, 
-                                             vjust=1)) +
-              labs(title= paste("File:",as.character(pop@call[2]),
-                                "\nN =",sum(mlgtab),"MLG =",length(mlgtab))
-                   ))
+      invisible(lapply(popnames, print_mlg_barplot, mlgtab, quiet=quiet))
+    } else {
+      print(mlg_barplot(mlgtab) + 
+        theme_classic() %+replace%
+        theme(axis.text.x=element_text(size=10, angle=-45, hjust=0, vjust=1)) +
+        labs(title = paste("File:", as.character(pop@call[2]), "\nN =",
+                           sum(mlgtab), "MLG =", length(mlgtab))))
     }
   }
-  #,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,#
-
   mlgtab <- mlgtab[, which(colSums(mlgtab) > 0)]
   return(mlgtab)
 }
 
 #==============================================================================#
 #' @rdname mlg
-# Multilocus Genotype Vector
-#
-# Create a vector of multilocus genotype indecies. 
-#
-# @param x a \code{\link{genind}} object.
-# 
-#' @return a numeric vector naming the multilocus genotype of each individual in
-#' the dataset. 
-#'
-#' @note The numbers of \code{mlg.vector} will not match up with the sequence of
-#' new genotypes found because sorting takes place within the algorithm before
-#' the genotypes are called so that the number of comparisons is \eqn{n-1} 
-#' instead of \eqn{\frac{n(n-1)}{2}}. 
-#' 
+#'   
+#' @note mlg.vector will recalculate the mlg vector for
+#'   \code{\linkS4class{genind}} objects and will return the contents of the mlg
+#'   slot in \code{\linkS4class{genclone}} objects. This means that MLGs will be
+#'   different for subsetted \code{\linkS4class{genind}} objects.
+#'   
 #' @export
-# @examples
-# mat1 <- matrix(ncol=5, 25:1)
-# mat1 <- rbind(mat1, mat1)
-# mat <- matrix(nrow=10, ncol=5, paste(mat1,mat1,sep="/"))
-# mat.gid <- df2genind(mat, sep="/")
-# mlg.vector(mat.gid)
-# mlg.table(mat.gid)
 #==============================================================================#
 
 mlg.vector <- function(pop){
@@ -297,13 +274,15 @@ mlg.vector <- function(pop){
   # Step 4: evaluate strings in sorted vector and increment to the respective 
   # # index vector each time a unique string occurs.
   # Step 4: Rearrange index vector with the indices from the original vector.
-
+  if (is.genclone(pop)){
+    return(pop@mlg)
+  }
   xtab <- pop@tab
   # concatenating each genotype into one long string.
   xsort <- vapply(seq(nrow(xtab)),function(x) paste(xtab[x, ]*pop@ploidy, 
-                                                    collapse=""), "string")
+                                                    collapse = ""), "string")
   # creating a new vector to store the counts of unique genotypes.
-  countvec <- vector(length=length(xsort), mode="numeric")
+  countvec <- vector(length = length(xsort), mode = "integer")
   # sorting the genotypes ($x) and preserving the index ($xi). 
   xsorted <- sort(xsort, index.return=TRUE)
   
@@ -314,17 +293,16 @@ mlg.vector <- function(pop){
   # object to modify. In this case it is countvec, which was declared above.
   
   f1 <- function(num, comp){
-    if(num-1 == 0){
-      countvec[num] <<- 1
+    if (num - 1 == 0){
+      countvec[num] <<- 1L
     }
     # These have the exact same strings, thus they are the same MLG. Perpetuate
     # The MLG index.
-    else if(comp[num] == comp[num-1]){
-      countvec[num] <<- countvec[num-1]
-    }
+    else if (comp[num] == comp[num - 1]){
+      countvec[num] <<- countvec[num - 1]
+    } else {
     # These have differnt strings, increment the MLG index by one.
-    else{
-      countvec[num] <<- countvec[num-1] + 1
+      countvec[num] <<- countvec[num - 1] + 1L
     }
   }
 
@@ -340,67 +318,73 @@ mlg.vector <- function(pop){
 }
 
 #==============================================================================#
-#' @rdname mlg
-# Multilocus Genotypes Across Populations
-#
-# Show which multilocus genotypes exist accross populations. 
-#
-# @param pop a \code{\link{genind}} object.
-# 
-#' @return a \code{list} containing vectors of population names for each MLG. 
+#' @rdname mlg 
 #' 
 #' @export
 #==============================================================================#
 
 mlg.crosspop <- function(pop, sublist="ALL", blacklist=NULL, mlgsub=NULL, indexreturn=FALSE, df=FALSE, quiet=FALSE){
-  if(length(sublist) == 1 & sublist[1] != "ALL" | is.null(pop(pop))){
+  if (length(sublist) == 1 & sublist[1] != "ALL" | is.null(pop(pop))){
     cat("Multiple populations are needed for this analysis.\n")
     return(0)
   }
-  vec <- mlg.vector(pop)
-  subind <- sub_index(pop, sublist, blacklist)
-  vec <- vec[subind]
-  mlgtab <- mlg.matrix(pop)
-  if(!is.null(mlgsub)){
-    mlgtab <- mlgtab[, mlgsub]
-    mlgs <- 1:ncol(mlgtab)
-    names(mlgs) <- colnames(mlgtab)
+  if (is.genclone(pop)){
+    vec <- pop@mlg
+  } else {
+    vec <- mlg.vector(pop) 
   }
-  else{
-    if(sublist[1] != "ALL" | !is.null(blacklist)){
-      pop <- popsub(pop, sublist, blacklist)
+  subind <- sub_index(pop, sublist, blacklist)
+  vec    <- vec[subind]
+  mlgtab <- mlg.matrix(pop)
+  if (!is.null(mlgsub)){
+    mlgsubnames <- paste("MLG", mlgsub, sep = ".")
+    matches <- mlgsubnames %in% colnames(mlgtab)
+    if (!all(matches)){
+      rejects <- mlgsub[!matches]
+      mlgsubnames  <- mlgsubnames[matches]
+      warning(mlg_sub_warning(rejects))
+    }
+    mlgtab <- mlgtab[, mlgsubnames, drop = FALSE]
+    mlgs   <- 1:ncol(mlgtab)
+    names(mlgs) <- colnames(mlgtab)
+  } else {
+    if (sublist[1] != "ALL" | !is.null(blacklist)){
+      pop    <- popsub(pop, sublist, blacklist)
       mlgtab <- mlgtab[unlist(vapply(pop@pop.names, 
-                  function(x) which(rownames(mlgtab)==x), 1)), , drop=FALSE]
+                  function(x) which(rownames(mlgtab) == x), 1)), , drop=FALSE]
     }
     #mlgtab <- mlgtab[, which(colSums(mlgtab) > 0)]
-    mlgs <- unlist(strsplit(names(which(colSums(ifelse(mlgtab==0, 0, 1)) > 1)), 
-                          "\\."))
-    mlgs <- as.numeric(mlgs[!mlgs %in% "MLG"])
-    if(length(mlgs) == 0){
+    # mlgs <- unlist(strsplit(names(which(colSums(ifelse(mlgtab == 0L, 0L, 1L)) > 1)), 
+    #                       "\\."))
+    # mlgs <- as.numeric(mlgs[!mlgs %in% "MLG"])
+    mlgs <- colSums(ifelse(mlgtab == 0L, 0L, 1L)) > 1
+    if (sum(mlgs) == 0){
       cat("No multilocus genotypes were detected across populations\n")
       return(0)
     }
-    names(mlgs) <- paste("MLG", mlgs, sep=".")
-    if(indexreturn){
-      return(mlgs)
+    #names(mlgs) <- paste("MLG", mlgs, sep=".")
+    if (indexreturn){
+      mlgout <- unlist(strsplit(names(mlgs[mlgs]), "\\."))
+      mlgout <- as.numeric(mlgout[!mlgout %in% "MLG"])
+      return(mlgout)
     }
   }
   popop <- function(x, quiet=TRUE){
-    popnames <- mlgtab[mlgtab[, x] > 0, x]
-    if(length(popnames) == 1){
-      names(popnames) <- rownames(mlgtab[mlgtab[, x] > 0, x, drop=FALSE])
+    popnames <- mlgtab[mlgtab[, x] > 0L, x]
+    if (length(popnames) == 1){
+      names(popnames) <- rownames(mlgtab[mlgtab[, x] > 0L, x, drop=FALSE])
     }
-    if(!quiet)
+    if (!quiet)
       cat(paste(x, ":", sep=""),paste("(",sum(popnames)," inds)", sep=""),
           names(popnames), fill=80)
     return(popnames)
   }
   # Removing any populations that are not represented by the MLGs.
-  mlgtab <- mlgtab[rowSums(mlgtab[, mlgs, drop=FALSE]) > 0, mlgs, drop=FALSE]
+  mlgtab <- mlgtab[rowSums(mlgtab[, mlgs, drop=FALSE]) > 0L, mlgs, drop=FALSE]
   # Compiling the list.
   mlg.dup <- lapply(colnames(mlgtab), popop, quiet=quiet)
   names(mlg.dup) <- colnames(mlgtab)
-  if(df == TRUE){
+  if (df == TRUE){
     mlg.dup <- as.data.frame(list(MLG = rep(names(mlg.dup), sapply(mlg.dup, length)), 
                              Population = unlist(lapply(mlg.dup, names)), 
                              Count = unlist(mlg.dup)))
@@ -408,3 +392,20 @@ mlg.crosspop <- function(pop, sublist="ALL", blacklist=NULL, mlgsub=NULL, indexr
   }
   return(mlg.dup)
 }
+
+
+
+#==============================================================================#
+#' @rdname mlg
+#' @export
+#==============================================================================#
+
+
+mlg.id <- function (pop){
+  if (!is.genind(pop)){
+    stop(paste(substitute(pop), "is not a genind or genclone object"))
+  }
+  ctab <- table(pop$ind.names, mlg.vector(pop))
+  m.g <- apply(ctab, MARGIN = 2, FUN = function (y) names(y[y > 0]))
+  return(m.g)
+}
