@@ -42,46 +42,45 @@
 #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!#
 #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!#
 #==============================================================================#
-#' Shuffle individuals in a \code{\link{genind}} object independently over each
-#' locus.
-#'
-#' @param pop a \code{\link{genind}} object
-#'
+#' Shuffle individuals in a \code{\linkS4class{genclone}} or
+#' \code{\linkS4class{genind}} object independently over each locus.
+#' 
+#' @param pop a \code{\linkS4class{genclone}} or \code{\linkS4class{genind}}
+#'   object
+#'   
 #' @param method an integer between 1 and 4. See details below.
-#'
-#' @return a \code{\link{genind}} object shuffled by a specified method
-#'
-#' @section Details: This function will shuffle each locus in the data set independently 
-#' of one another, rendering them essentially unlinked. The following methods
-#' are available to shuffle your data:
-#' \enumerate{ 
-#'  \item \strong{Permute Alleles} This will redistribute all alleles in the
-#' sample throughout the locus. Missing data is fixed in place. This maintains
-#' allelic structure, but heterozygosity is variable.
-#'  \item \strong{Parametric Bootstrap} This will redistribute available alleles
-#' within the locus based on their allelic frequencies. This means that both the
-#' allelic state and heterozygosity will vary. The resulting data set will not
-#' have missing data.
-#'  \item \strong{Non-Parametric Bootstrap} This will shuffle the
-#' allelic state for each individual. Missing data is fixed in place.
-#'  \item \strong{Multilocus Style Permutation} This will shuffle the genotypes 
-#' at each locus, maintaining the heterozygosity and allelic structure.
-#' } 
-#'
-#'
+#'   
+#' @return a \code{\linkS4class{genclone}} or \code{\linkS4class{genind}} object
+#'   shuffled by a specified method
+#'   
+#' @details This function will shuffle each locus in the data set independently 
+#'   of one another, rendering them essentially unlinked. The following methods 
+#'   are available to shuffle your data: \enumerate{ \item \strong{Permute 
+#'   Alleles} This will redistribute all alleles in the sample throughout the 
+#'   locus. Missing data is fixed in place. This maintains allelic structure, 
+#'   but heterozygosity is variable. \item \strong{Parametric Bootstrap} This 
+#'   will redistribute available alleles within the locus based on their allelic
+#'   frequencies. This means that both the allelic state and heterozygosity will
+#'   vary. The resulting data set will not have missing data. \item 
+#'   \strong{Non-Parametric Bootstrap} This will shuffle the allelic state for 
+#'   each individual. Missing data is fixed in place. \item \strong{Multilocus 
+#'   Style Permutation} This will shuffle the genotypes at each locus, 
+#'   maintaining the heterozygosity and allelic structure. }
+#'   
+#'   
 #' @export
 #' @author Zhian N. Kamvar
-#'
-#' @references  Paul-Michael Agapow and Austin Burt. 2001. Indices of multilocus 
-#' linkage disequilibrium. \emph{Molecular Ecology Notes}, 1(1-2):101-102 
-#'
+#'   
+#' @references  Paul-Michael Agapow and Austin Burt. 2001. Indices of multilocus
+#'   linkage disequilibrium. \emph{Molecular Ecology Notes}, 1(1-2):101-102
+#'   
 #' @examples
 #' # load the microbov dataset
 #' data(microbov)
 #' # Let's look at a single population for now. Howsabout Zebu
 #' Zebu <- popsub(microbov, "Zebu")
 #' summary(Zebu)
-#'
+#' 
 #' # Take note of the Number of alleles per population and the Observed
 #' # heterozygosity as we go through each method.
 #' 
@@ -90,7 +89,7 @@
 #' \dontrun{
 #' # Parametric Bootstrap: do not maintain allelic state or heterozygosity
 #' summary(shufflepop(Zebu, method=2))
-#'
+#' 
 #' # Non-Parametric Bootstrap: do not maintain allelic state or heterozygosity.
 #' summary(shufflepop(Zebu, method=3))
 #' 
@@ -163,14 +162,29 @@ shufflefunk <- function(pop, FUN, sample=1, method=1, ...){
                                  )
                             )
   if(!quiet) progbar <- txtProgressBar(style = 3)
-	for (c in 1:iterations){
-    IarD <- .Ia.Rd(.all.shuffler(pop, type, method=method), missing=missing)
+  # ploid <- ploidy(pop[[1]])
+  # tz <- test_zeroes(pop[[1]])
+  # if (ploid > 2 & tz){
+  #   zcol <- which(as.numeric(pop[[1]]@all.names[[1]]) == 0)
+  #   ploidvec <- vapply(pop, get_local_ploidy, integer(nInd(pop[[1]])))
+  #   ploidind <- TRUE
+  # } else {
+  #   ploidind <- FALSE
+  # }
+  for (c in 1:iterations){
+    # if (ploidind){
+    #   IarD <- .Ia.Rd(new.all.shuff(pop, type, method, ploidvec = ploidvec, 
+    #                                zerocol = zcol), missing = missing)
+    # } else {
+      IarD <- .Ia.Rd(.all.shuffler(pop, type, method=method), missing=missing)
+    # }
     sample.data$Ia[c]    <- IarD[1]
     sample.data$rbarD[c] <- IarD[2]
     if (!quiet){
       setTxtProgressBar(progbar, c/iterations)
     }
   }
+
   if(!quiet) close(progbar)
 	return(sample.data)
 }
@@ -260,7 +274,7 @@ shufflefunk <- function(pop, FUN, sample=1, method=1, ...){
 #
 # weights is a corresponding vector giving the allelic frequency for each allele
 # at that locus in that population. The sum of the frequencies should be 1. 
-# DEPRECIATED (replaced with multinomial distribution)
+# DEPRECATED (replaced with multinomial distribution)
 #==============================================================================# 
 .diploid.shuff <- function(vec, weights){
   # Dealing with missing values is probably not necessary for a parametric
@@ -296,9 +310,9 @@ shufflefunk <- function(pop, FUN, sample=1, method=1, ...){
 #==============================================================================#
   
 .permut.shuff <- function(mat, ploidy = 2){
-  bucket     <- colSums(mat, na.rm = TRUE)*ploidy
+  bucket     <- round(colSums(mat, na.rm = TRUE)*ploidy)
   bucketlist <- as.integer(sample(rep(1:length(bucket), bucket)))
-  mat        <- .Call("permute_shuff", mat, bucketlist - 1, 1/ploidy, ploidy)
+  mat        <- .Call("permute_shuff", mat, bucketlist - 1, 1/ploidy, ploidy, PACKAGE = "poppr")
   return(mat)
 }
 
