@@ -46,8 +46,8 @@ double bruvo_dist(int *in, int *nall, int *perm, int *woo);
 double test_bruvo_dist(int *in, int *nall, int *perm, int *woo, int *loss, int *add);
 void permute(int *a, int i, int n, int *c);
 int fact(int x);
-double mindist(int perms, int alleles, int *perm, double *dist);
-void genome_add_calc(int perms, int alleles, int *perm, double *dist, 
+double mindist(int perms, int alleles, int *perm, double **dist);
+void genome_add_calc(int perms, int alleles, int *perm, double **dist, 
 	int zeroes, int *zero_ind, int curr_zero, int miss_ind, int *replacement, 
 	int inds, int curr_ind, double *genome_add_sum, int *tracker);
 void genome_loss_calc(int *genos, int nalleles, int *perm_array, int woo, 
@@ -58,73 +58,10 @@ void fill_short_geno(int *genos, int nalleles, int *perm_array, int *woo,
 		int *loss, int *add, int zeroes, int *zero_ind, int curr_zero, 
 		int miss_ind, int *replacement, int inds, int curr_ind, double *res, 
 		int *tracker);
+void print_distmat(double** dist, int* genos, int p);
 		
 		
-		
-SEXP raw_pairdiffs(SEXP mat, SEXP ploidy)
-{
-	char binary_diffs, homozygote;
-	int count, row, col, bitcount, hz, a1, a2, ht;//, derp, i, j, k;
-	//SEXP Rout;
-	SEXP Rdim;
-	//SEXP dvector;
-	//SEXP Dvector;
-	Rdim = getAttrib(mat, R_DimSymbol);
-	row = INTEGER(Rdim)[0];
-	col = INTEGER(Rdim)[1];
-	/*
-	ploidy = coerceVector(ploidy, INTSXP);
-	PROTECT(dvector = allocVector(dvector, col));
-	PROTECT(Dvector = allocVector(Dvector, ((row*(row-1)/2))));
-	//mat = coerceVector(mat, RAWSXP);
-	for(i = 0; i < row - 1; i+ploidy)
-	{
-		for(j = i+ploidy; j < row; j+ploidy)
-		{
-			count = 0;
-			for(k = 0; k < col; k++)
-			{
-				binary_diffs = RAW(mat)[i+col*ploidy+k] ^ RAW(mat)[j+col*ploidy+k];
-				for(bitcount = 7; bitcount >= 0; bitcount--)
-				{
-					dvector[count] += (binary_diffs >> bitcount) & 0x01;
-				}
-				count++;
-			}
-		}
-	}
-*/
 
-	for(count = 0; count < row*col; count++)
-	{
-		if(count < (row*col)-1 && count % 2 == 0)
-		{
-			binary_diffs = RAW(mat)[count] ^ RAW(mat)[count+1];
-			homozygote = RAW(mat)[count] & RAW(mat)[count+1];
-		}
-		else
-		{
-			goto out;
-		}
-		for(bitcount = 7; bitcount >= 0; bitcount--)
-		{
-			//printf("Genotype:\t\t%d\n", (RAW(mat)[count] >> bitcount) & 0x01);
-			if(count < (row*col)-1)
-			{
-				hz = (homozygote >> bitcount) & 0x01;
-				a1 = (RAW(mat)[count] >> bitcount) & 0x01;
-				a2 = (RAW(mat)[count + 1] >> bitcount) & 0x01;
-				ht = (binary_diffs >> bitcount) & 0x01;
-				Rprintf("%d AND %d:\t%d\t\t", a1, a2, hz);
-				Rprintf("%d XOR %d:\t%d\t\t", a1, a2, ht);
-				Rprintf("RESULT:\t%d\n", hz+ht);						
-			}
-		}
-		out:
-		Rprintf("\n");
-	}
-	return R_NilValue;
-}
 /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 Calculates the root product of pairwise comparisons of each of the variances of
 each locus.
@@ -134,7 +71,10 @@ Output: A vector of length n*(n-1)/2
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
 SEXP pairwise_covar(SEXP pair_vec)
 {
-	int I, i, j, count;
+	int I;
+	int i;
+	int j;
+	int count;
 	SEXP Rout;
 	I = length(pair_vec);
 	pair_vec = coerceVector(pair_vec, REALSXP);
@@ -164,7 +104,12 @@ Output: A vector of length n*(n-1)/2
 SEXP pairdiffs(SEXP freq_mat)
 {
 
-	int I, J, i, j, z, count;
+	int I;
+	int J;
+	int i;
+	int j;
+	int z;
+	int count;
 	double P;
 	SEXP Rout;
 	SEXP Rdim;
@@ -204,7 +149,9 @@ permuto will return a vector of all permutations needed for bruvo's distance.
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
 SEXP permuto(SEXP perm)
 {
-	int permutations, i, per;
+	int permutations;
+	int i;
+	int per;
 	SEXP Rval;
 	/* 
 		IMPORTANT: INITIALIZE THE COUNTER. THE POINTER IS NOT RELEASED FROM
@@ -213,7 +160,8 @@ SEXP permuto(SEXP perm)
 	perm_count = 0;
 	perm = coerceVector(perm, INTSXP);
 	per = INTEGER(perm)[0];
-	int allele_array[per];
+	int *allele_array;
+	allele_array = R_Calloc(per, int);
 	permutations = fact(per)*per;
 	for(i = 0; i < per; i++)
 	{
@@ -222,6 +170,7 @@ SEXP permuto(SEXP perm)
 	PROTECT(Rval = allocVector(INTSXP, permutations));
 	permute(allele_array, 0, per-1, INTEGER(Rval));
 	UNPROTECT(1);
+	R_Free(allele_array);
 	return Rval;
 }
 /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -247,7 +196,7 @@ with missing data. In the wrapping R function, 100s will be converted to NAs
 and then the average over all loci will be taken. 
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
 
-SEXP single_bruvo(SEXP b_mat, SEXP permutations, SEXP alleles, SEXP loss, SEXP add)
+SEXP single_bruvo(SEXP b_mat, SEXP permutations, SEXP alleles, SEXP add, SEXP loss)
 {
 	int A, P, *pA, *pP;
 	SEXP Rval;
@@ -269,7 +218,7 @@ SEXP single_bruvo(SEXP b_mat, SEXP permutations, SEXP alleles, SEXP loss, SEXP a
     
 }
 
-SEXP bruvo_distance(SEXP bruvo_mat, SEXP permutations, SEXP alleles, SEXP m_loss, SEXP m_add)
+SEXP bruvo_distance(SEXP bruvo_mat, SEXP permutations, SEXP alleles, SEXP m_add, SEXP m_loss)
 {
 	/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 	I = number of rows in bruvo_mat
@@ -408,11 +357,8 @@ int fact(int x)
 
 /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-	DEPRECATED
-
-	This will calculate bruvo's distance between two individuals. 
-	All that needs to be done from here is to have it do the pairwise
-	calculations. 
+	Bruvo's Distance implementing Addition, Loss, and Infinite models for
+	imputation of missing data.
 
 	NOTE: The input needs to be divided by the repeat length beforehand for this
 	to work. 
@@ -420,68 +366,13 @@ int fact(int x)
 	in: a matrix of two individuals
 	out: a double value that will be the output from bruvo's distance.
 	n: number of individuals(2)
-	nall / p: number of alleles
+	nall: number of alleles
 	perm: a vector from the permn function in R
 	woo: p * p!
-	minn: is a rolling counter of the minimum between allele compairsons.
+	loss: TRUE/FALSE: impute under genome loss model.
+	add: TRUE/FALSE: impute under genome addition model. 
 
-~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
-double bruvo_dist(int *in, int *nall, int *perm, int *woo)
-{
-	int i, j, counter=0, n = 2, p = *nall, w = *woo, genos[2][p];
-	double dist[p][p], da, res = 0, minn=100;
-	// reconstruct the genotype table.
-	for(i=0; i < n; i++)
-	{
-		for(j = 0; j < p; j++)
-		{
-			// Missing data will return with distance of 100
-			if(in[counter] == 0)
-			{
-				return minn;
-			}
-			else
-			{
-				genos[i][j] = in[counter++];
-			}
-		}
-	}
-
-	// Construct distance matrix of 1 - 2^{-|x|}
-	for(j = 0; j < p; j++)
-	{
-		for(i=0; i < p; i++)
-		{
-			da = 1- pow(2 , -abs(genos[0][i] - genos[1][j]));
-			dist[i][j] = da;
-		}
-	}
-
-	//	Calculate the smallest s, which is the minimum distance among alleles.
-	for(i = 0; i < w; i += p)
-	{
-		for(j = 0; j < p; j++)
-		{
-			if (j == 0)
-			{
-				res = dist[*perm++][j];
-			}
-			else
-			{
-				res += dist[*perm++][j];
-			}
-		}
-		/*	Checking if the new calculated distance is smaller than the smallest
-			distance seen. */
-		if ( res < minn )
-		{
-			minn = res;
-		}
-	}
-	return minn/p;
-}
-
-/*	Test code comparing current status to polysat's Bruvo2.distance:
+	Test code comparing current status to polysat's Bruvo2.distance:
 ================================================================================
 poppr_bruvo <- function(){ 
   return(c(.Call("single_bruvo", c(20,23,24,0,20,24,26,43), .Call("permuto", 4), 4, 0, 0),
@@ -493,9 +384,9 @@ poppr_bruvo <- function(){
 
 polysat_bruvo <- function(){
   return(c(Bruvo2.distance(c(20,23,24), c(20,24,26,43), usatnt=1, loss=FALSE, add=FALSE),
-Bruvo2.distance(c(20,23,24), c(20,24,26,43), usatnt=1, loss=T, add=FALSE),
-Bruvo2.distance(c(20,23,24), c(20,24,26,43), usatnt=1, loss=F, add=T),
-Bruvo2.distance(c(20,23,24), c(20,24,26,43), usatnt=1, loss=T, add=T)
+Bruvo2.distance(c(20,23,24), c(20,24,26,43), usatnt=1, loss=FALSE, add=TRUE),
+Bruvo2.distance(c(20,23,24), c(20,24,26,43), usatnt=1, loss=TRUE, add=FALSE),
+Bruvo2.distance(c(20,23,24), c(20,24,26,43), usatnt=1, loss=TRUE, add=TRUE)
 ))
 }
 
@@ -506,10 +397,40 @@ polysat_bruvo() == poppr_bruvo()
 ==============================================================================*/
 double test_bruvo_dist(int *in, int *nall, int *perm, int *woo, int *loss, int *add)
 {
-	int i, j, counter = 0, n = 2, p = *nall, w = *woo, loss_indicator = *loss, 
-		add_indicator = *add, genos[2][p], zerocatch[2], zero_ind[2][p],
-		zerodiff;
-	double dist[p][p], da, minn = 100, *distp;
+	int i; 
+	int j; 
+	int counter = 0;    // counter used for building arrays
+	int n = 2;          // number of individuals
+	int p = *nall;      // number of alleles
+	int w = *woo;       // number of permutations = p * p!
+	int loss_indicator = *loss; // 1 if the genome loss model should be used
+	int add_indicator = *add;   // 1 if the genome addition model should be used
+
+	int *genos;    // array to store the genotypes
+	genos = R_Calloc(2*p, int);
+
+	int zerocatch[2]; 	// 2 element array to store the number of missing 
+					  	// alleles in each genotype. 
+	 
+	int** zero_ind;     // array to store the indices of the missing data for
+						// each genotype
+	zero_ind = R_Calloc(2, int*);
+	zero_ind[0] = R_Calloc(p, int);
+	zero_ind[1] = R_Calloc(p, int);
+
+	int zerodiff;      	// used to check the amount of missing data different 
+				       	// between the two genotypes
+	
+
+	double** dist; 	    // array to store the distance
+	dist = R_Calloc(p, double*);
+	for (i = 0; i < p; i++)
+	{
+		dist[i] = R_Calloc(p, double);
+	}
+
+	double minn = 100; 	// The minimum distance 
+
 	// reconstruct the genotype table.
 	zerocatch[0] = 0;
 	zerocatch[1] = 0;
@@ -518,22 +439,22 @@ double test_bruvo_dist(int *in, int *nall, int *perm, int *woo, int *loss, int *
 		for(j = 0; j < p; j++)
 		{
 			// Catch missing data here.
-			if(in[counter] == 0)
+			if (in[counter] == 0)
 			{
 				if (zerocatch[i] == p - 1)
 				{
 					return minn;
 				}
 				zerocatch[i] += 1;
-				//printf("#");
+				// Rprintf("[%d]", zerocatch[i] - 1);
 				zero_ind[i][zerocatch[i] - 1] = j;
 			}
-			genos[i][j] = in[counter++];
-			//printf("%d\t", genos[i][j]);
+			genos[i*p + j] = in[counter++];
+			// Rprintf("%d\t", genos[i*p + j]);
 		}
-		//printf("\n");
+		// Rprintf("\n");
 	}
-	//printf("\n");
+	// Rprintf("\n");
 	zerodiff = abs(zerocatch[0] - zerocatch[1]);
 	/*==========================================================================
 	* Removing superfluous zeroes from the data. This is in the case that both
@@ -543,8 +464,15 @@ double test_bruvo_dist(int *in, int *nall, int *perm, int *woo, int *loss, int *
 	==========================================================================*/
 	if (zerocatch[0] > 0 && zerocatch[1] > 0)
 	{
-		int smaller = 0, larger = 1, reduction = 0, i, j,
-			zero_counter, *perm_array, *new_genop;
+		int smaller = 0;
+		int larger = 1;
+		int reduction = 0; 
+		int i;
+		int j;
+		int zero_counter;
+		int *perm_array;
+		int *new_geno;
+		int *new_alleles;
 
 		if (zerodiff == 0)
 		{
@@ -552,114 +480,87 @@ double test_bruvo_dist(int *in, int *nall, int *perm, int *woo, int *loss, int *
 		}
 		else
 		{
-			if(zerocatch[0] < zerocatch[1])
+			if (zerocatch[0] < zerocatch[1])
 			{
 				smaller = 1;
 				larger = 0;
 			}
 			reduction = p - (zerocatch[smaller] - zerodiff);
 		}
-		int del = 1;
-		if (del > 0)
-		{		
-			int new_alleles[reduction];
-			for (i = 0; i < reduction; i++)
-			{
-				new_alleles[i] = i;
-			}
-			w = fact(reduction) * reduction;
-			perm_array = (int *) malloc(w * sizeof(int));
-			perm_count = 0;
-			permute(new_alleles, 0, reduction - 1, perm_array);
-			// rebuild the array and make a pointer.
-			int new_geno[reduction*n];
-			counter = 0;
-			for (i=0; i < n; i++)
-			{
-				zero_counter = zerocatch[larger];
-				for (j = 0; j < p; j++)
-				{
-					if (genos[i][j] == 0 && zero_counter > 0)
-					{
-						zero_counter--;
-					}
-					else
-					{
-						new_geno[counter++] = genos[i][j];
-					}
-				}
-			}
-			new_genop = (int *) &new_geno;
-		
-			minn = test_bruvo_dist(new_genop, &reduction, perm_array, &w, 
-											&loss_indicator, &add_indicator);
-			free(perm_array);
-		}
-
-		/*	Questions of the proper way of permuting this arise: 
-		*	1.	If both of the genotypes are of equal length, but the user wants
-		*		a non genome - addition model, how do we undertake that?
-		*		should we simply run all combinations of both genotypes
-		*		separately?
-		*	2.	If one genotype is longer than the other, should we fill the
-		*		larger or smaller genotype, or possibly, should we fill both and
-		* 		do a similar procedure as I described above? 
-
-
-		else
+	
+		new_alleles = R_Calloc(reduction, int);
+		for (i = 0; i < reduction; i++)
 		{
-			int fill_tracker = 0, *pzero_ind, large_inds[p - zerocatch[larger]], 
-				large_counter = 0, *plarge_inds;
-			double res = 0;
-			pzero_ind = (int *) &zero_ind[larger];
-			plarge_inds = (int *) &large_inds;
-			for (i = 0; i < p; i++)
+			new_alleles[i] = i;
+		}
+		w = fact(reduction) * reduction;
+		perm_array = R_Calloc(w, int);
+		perm_count = 0;
+		permute(new_alleles, 0, reduction - 1, perm_array);
+		new_geno = R_Calloc(reduction*n, int);
+		counter = 0;
+		for (i=0; i < n; i++)
+		{
+			zero_counter = zerocatch[larger];
+			for (j = 0; j < p; j++)
 			{
-				if (genos[larger][i] > 0)
+				if (genos[i*p + j] == 0 && zero_counter > 0)
 				{
-					large_inds[large_counter++] = i;
+					zero_counter--;
+				}
+				else
+				{
+					new_geno[counter++] = genos[i*p + j];
 				}
 			}
-			for (i = 0; i < reduction; i++)
-			{
-				fill_short_geno(in, p, perm, woo, loss, add, zerocatch[larger], 
-					pzero_ind, 0, larger, plarge_inds, reduction, i, &res, 
-					&fill_tracker);
-			}
-			minn = res/fill_tracker;
 		}
-    */
+	
+		minn = test_bruvo_dist(new_geno, &reduction, perm_array, &w, 
+								&loss_indicator, &add_indicator);
+		R_Free(perm_array);
+		R_Free(new_geno);
+		R_Free(new_alleles);
 		return minn;
 	}
 
 	// Construct distance matrix of 1 - 2^{-|x|}.
-	// This is constructed column by column. Genotype 1 in the rows. Genotype 2
-	// in the columns.
-	for(j = 0; j < p; j++)
+	// This is constructed column by column. 
+	// Genotype 1: COLUMNS
+	// Genotype 2: ROWS
+
+	for(i = 0; i < p; i++)
 	{
-		for(i = 0; i < p; i++)
+		for(j = 0; j < p; j++)
 		{
-			da = 1 - pow(2, -abs(genos[0][i] - genos[1][j]));
-			dist[i][j] = da;
+			dist[i][j] = 1 - pow(2, -abs(genos[0*p + j] - genos[1*p + i]));
 		}
 	}
-	// This avoids warning: assignment from incompatible pointer type
-	distp = (double *) &dist;
-	if(zerocatch[0] > 0 || zerocatch[1] > 0)
+
+	// Rprintf("DISTANCE:");
+	// print_distmat(dist, genos, p);
+
+	if (zerocatch[0] > 0 || zerocatch[1] > 0)
 	{
-		int *genop, ind, miss_ind = 1, z, tracker = 0, loss_tracker = 0;// full_ind = 0;
-		double genome_add_sum = 0, genome_loss_sum = 0;
-		genop = (int *) &genos;
-		if (zerocatch[0] > 0) // The rows contain the zero value
+		int ind;    
+		int miss_ind = 1;
+		int z;
+		int tracker = 0;
+		int loss_tracker = 0;
+		int comparison_factor = 1;
+		int* short_inds;
+		double genome_add_sum = 0;
+		double genome_loss_sum = 0;
+
+		if (zerocatch[0] > 0) // The columns contain the zero value
 		{
 			miss_ind = 0;
-			//full_ind = 1; 
 		}
 		ind = zero_ind[miss_ind][0];
-		int short_inds[zerocatch[miss_ind]], short_counter = 0;
+		short_inds = R_Calloc(zerocatch[miss_ind], int);
+		int short_counter = 0;
 		for (i = 0; i < p; i++)
 		{
-			if (genos[miss_ind][i] > 0)
+			if (genos[miss_ind*p + i] > 0)
 			{
 				short_inds[short_counter++] = i;
 			}
@@ -671,10 +572,11 @@ double test_bruvo_dist(int *in, int *nall, int *perm, int *woo, int *loss, int *
 		======================================================================*/
 		if(loss_indicator != 1 && add_indicator != 1)
 		{
+			// Rprintf("TO INFINITY!\n");
 			for (z = 0; z < zerocatch[miss_ind]; z++)
 			{
 				ind = zero_ind[miss_ind][z];
-				if (zerocatch[0] > 0)
+				if (zerocatch[0] < 0)
 				{
 					for (j = 0; j < p; j++)
 					{
@@ -689,7 +591,7 @@ double test_bruvo_dist(int *in, int *nall, int *perm, int *woo, int *loss, int *
 					}						
 				}
 			}
-			return mindist(w, p, perm, distp)/p;
+			goto finalcalc;
 		}
 		/*======================================================================
 		*	GENOME ADDITION MODEL
@@ -700,15 +602,18 @@ double test_bruvo_dist(int *in, int *nall, int *perm, int *woo, int *loss, int *
 		======================================================================*/
 		if (add_indicator == 1)
 		{
-			int *pzero_ind, *pshort_inds;
-			pzero_ind = (int *) &zero_ind[miss_ind];
-			pshort_inds = (int *) &short_inds;
-			for (i = 0; i < p - zerocatch[miss_ind]; i++)
+			// Rprintf("ADD!\n");
+			int replacements = 0;
+			replacements = p - zerocatch[miss_ind];
+			// print_distmat(dist, genos, p);
+			for (i = 0; i < replacements; i++)
 			{
-				genome_add_calc(w, p, perm, distp, zerocatch[miss_ind],
-					pzero_ind, 0, miss_ind, pshort_inds, p-zerocatch[miss_ind], 
+				genome_add_calc(w, p, perm, dist, zerocatch[miss_ind],
+					zero_ind[miss_ind], 0, miss_ind, short_inds, replacements, 
 					i, &genome_add_sum, &tracker);
+				// Rprintf("current add sum = %.6f\n", genome_add_sum);
 			}
+			R_Free(short_inds);
 		}
 		/*======================================================================
 		*	GENOME LOSS MODEL
@@ -718,12 +623,11 @@ double test_bruvo_dist(int *in, int *nall, int *perm, int *woo, int *loss, int *
 		======================================================================*/
 		if (loss_indicator == 1)
 		{
-			int *pzero_ind;
-			pzero_ind = (int *) &zero_ind[miss_ind];
+			// Rprintf("LOSS!\n");
 			for (i = 0; i < p; i++)
 			{
-				genome_loss_calc(genop, p, perm, w, &loss_indicator, 
-					&add_indicator, pzero_ind, 0, zerocatch[miss_ind], 
+				genome_loss_calc(genos, p, perm, w, &loss_indicator, 
+					&add_indicator, zero_ind[miss_ind], 0, zerocatch[miss_ind], 
 					miss_ind, i, &genome_loss_sum, &loss_tracker);
 			}
 		}
@@ -736,11 +640,26 @@ double test_bruvo_dist(int *in, int *nall, int *perm, int *woo, int *loss, int *
 			loss_tracker = 1;
 		}
 		genome_loss_sum = genome_loss_sum/loss_tracker;
+		// Rprintf("LOSS SUM: %.6f\n", genome_loss_sum/p);
 		genome_add_sum = genome_add_sum/tracker;
-		int comparison_factor = loss_indicator + add_indicator;
-		return (genome_add_sum + genome_loss_sum)/(p*comparison_factor);
+		// Rprintf("ADD SUM: %.6f\n", genome_add_sum/p);
+		comparison_factor = loss_indicator + add_indicator;
+		minn = (genome_add_sum + genome_loss_sum)/(p*comparison_factor);
 	}
-	return mindist(w, p, perm, distp)/p;
+	else 
+	{
+		finalcalc: minn = mindist(w, p, perm, dist)/p;
+	}
+	R_Free(genos);
+	for (i = 0; i < p; i++)
+	{
+		R_Free(dist[i]);
+	}
+	R_Free(dist);
+	R_Free(zero_ind[0]);
+	R_Free(zero_ind[1]);
+	R_Free(zero_ind);
+	return minn;
 }
 
 
@@ -755,7 +674,7 @@ double test_bruvo_dist(int *in, int *nall, int *perm, int *woo, int *loss, int *
 *	perms - number of permutations (passed to mindist)
 *	alleles - number of maximum alleles (also passed to mindist)
 *	*perm - permutation array (passed to mindist)
-*	*dist - distance matrix to be manipulated (also passed to mindist)
+*	**dist - distance matrix to be manipulated (also passed to mindist)
 *	
 *	zeroes - number of zeroes present in the shorter genotype.
 *	*zero_ind - array containing indices of the zero values of the short geno
@@ -769,31 +688,30 @@ double test_bruvo_dist(int *in, int *nall, int *perm, int *woo, int *loss, int *
 *	*genome_add_sum - pointer to the total number of the genome addition model.
 *	*tracker - pointer to counter for the number of calculations for addition. 
 ==============================================================================*/
-void genome_add_calc(int perms, int alleles, int *perm, double *dist, 
+void genome_add_calc(int perms, int alleles, int *perm, double **dist, 
 	int zeroes, int *zero_ind, int curr_zero, int miss_ind, int *replacement, 
 	int inds, int curr_ind, double *genome_add_sum, int *tracker)
 {
-	int i,j;
+	int i;
+	int j;
 	//==========================================================================
 	// Part 1: fill one row/column of the matrix.
 	// Note that we don't have the format of the 2D array here, so we are
 	// cheating a little bit. Instead of indexing by dist[i][j] over p columns,
-	// we use dist[i + p*j]. It works. 
+	// we use dist[i][j]. It works. 
 	//==========================================================================
 	if(miss_ind > 0)
 	{
 		for (j = 0; j < alleles; j++)
 		{
-			dist[zero_ind[curr_zero] + alleles*j] = 
-				dist[replacement[curr_ind] + alleles*j];
+			dist[zero_ind[curr_zero]][j] = dist[replacement[curr_ind]][j];
 		}
 	}
 	else
 	{
 		for (j = 0; j < alleles; j++)
 		{
-			dist[j + alleles*zero_ind[curr_zero]] = 
-				dist[j + alleles*replacement[curr_ind]];	
+			dist[j][zero_ind[curr_zero]] = dist[j][replacement[curr_ind]];	
 		}
 	}
 	//==========================================================================
@@ -864,7 +782,8 @@ void genome_loss_calc(int *genos, int nalleles, int *perm_array, int woo,
 		int miss_ind, int curr_allele, double *genome_loss_sum, 
 		int *loss_tracker)
 {
-	int i, full_ind;
+	int i; 
+	int full_ind;
 	full_ind = 1 + (0 - miss_ind);
 	genos[miss_ind*nalleles + zero_ind[curr_zero]] = 
 		genos[full_ind*nalleles + curr_allele];
@@ -951,17 +870,23 @@ void fill_short_geno(int *genos, int nalleles, int *perm_array, int *woo,
 // Multiset coefficient: fact(n+k-1)/(fact(k)*fact(n-1))
 */
 
-double mindist(int perms, int alleles, int *perm, double *dist)
+double mindist(int perms, int alleles, int *perm, double **dist)
 {
-	int i, j, w = perms, p = alleles, counter = 0;
-	double res = 0, minn = 100;
+	int i, j;
+	int w = perms;
+	int p = alleles;
+	int counter = 0;
+	double res = 0;
+	double minn = 100;
+
 	for(i = 0; i < w; i += p)
 	{
 		for(j = 0; j < p; j++)
 		{
 			if (j == 0)
 			{
-				res = dist[*(perm + counter++) + p*j];
+				res = dist[*(perm + counter)][j];
+				counter++;
 				if(res > minn)
 				{
 					j = p;
@@ -971,7 +896,8 @@ double mindist(int perms, int alleles, int *perm, double *dist)
 			}
 			else
 			{
-				res += dist[*(perm + counter++) + p*j];
+				res += dist[*(perm + counter)][j];
+				counter++;
 				if(j < p-1 && res > minn)
 				{				
 					counter += (p-j-1);
@@ -988,3 +914,27 @@ double mindist(int perms, int alleles, int *perm, double *dist)
 	}
 	return minn;
 }
+
+void print_distmat(double** dist, int* genos, int p)
+{
+	int i;
+	int j;
+
+	Rprintf("\n \t");
+	for (i = 0; i < p; i++)
+	{
+		Rprintf("%d\t", genos[i]);
+	}
+	Rprintf("\n");
+	for (i = 0; i < p; i++)
+	{
+		Rprintf("%d\t", genos[i+p]);
+		for (j = 0; j < p; j++)
+		{
+			Rprintf("%.4f\t", dist[i][j]);
+		}
+		Rprintf("\n");
+	}
+	return;
+}
+
